@@ -1,18 +1,33 @@
 import datetime
 import platform
 import re
+from unittest import mock
 
 import contourpy
 import numpy as np
 from numpy.testing import (
-    assert_array_almost_equal, assert_array_almost_equal_nulp)
+    assert_array_almost_equal, assert_array_almost_equal_nulp, assert_array_equal)
 import matplotlib as mpl
-from matplotlib.testing.decorators import image_comparison
 from matplotlib import pyplot as plt, rc_context, ticker
 from matplotlib.colors import LogNorm, same_color
+import matplotlib.patches as mpatches
+from matplotlib.testing.decorators import check_figures_equal, image_comparison
 import pytest
 
 
+# Helper to test the transition from ContourSets holding multiple Collections to being a
+# single Collection; remove once the deprecated old layout expires.
+def _maybe_split_collections(do_split):
+    if not do_split:
+        return
+    for fig in map(plt.figure, plt.get_fignums()):
+        for ax in fig.axes:
+            for coll in ax.collections:
+                if isinstance(coll, mpl.contour.ContourSet):
+                    with pytest.warns(mpl._api.MatplotlibDeprecationWarning):
+                        coll.collections
+
+
 def test_contour_shape_1d_valid():
 
     x = np.arange(10)
@@ -85,8 +100,17 @@ def test_contour_Nlevels():
     assert (cs1.levels == cs2.levels).all()
 
 
-@image_comparison(['contour_manual_labels'], remove_text=True, style='mpl20')
-def test_contour_manual_labels():
+@check_figures_equal(extensions=['png'])
+def test_contour_set_paths(fig_test, fig_ref):
+    cs_test = fig_test.subplots().contour([[0, 1], [1, 2]])
+    cs_ref = fig_ref.subplots().contour([[1, 0], [2, 1]])
+
+    cs_test.set_paths(cs_ref.get_paths())
+
+
+@pytest.mark.parametrize("split_collections", [False, True])
+@image_comparison(['contour_manual_labels'], remove_text=True, style='mpl20', tol=0.26)
+def test_contour_manual_labels(split_collections):
     x, y = np.meshgrid(np.arange(0, 10), np.arange(0, 10))
     z = np.max(np.dstack([abs(x), abs(y)]), 2)
 
@@ -97,9 +121,12 @@ def test_contour_manual_labels():
     pts = np.array([(2.0, 3.0), (2.0, 4.4), (2.0, 6.0)])
     plt.clabel(cs, manual=pts, fontsize='small', colors=('r', 'g'))
 
+    _maybe_split_collections(split_collections)
 
+
+@pytest.mark.parametrize("split_collections", [False, True])
 @image_comparison(['contour_manual_colors_and_levels.png'], remove_text=True)
-def test_given_colors_levels_and_extends():
+def test_given_colors_levels_and_extends(split_collections):
     # Remove this line when this test image is regenerated.
     plt.rcParams['pcolormesh.snap'] = False
 
@@ -128,10 +155,12 @@ def test_given_colors_levels_and_extends():
 
         plt.colorbar(c, ax=ax)
 
+    _maybe_split_collections(split_collections)
 
-@image_comparison(['contour_log_locator.svg'], style='mpl20',
-                  remove_text=False)
-def test_log_locator_levels():
+
+@pytest.mark.parametrize("split_collections", [False, True])
+@image_comparison(['contour_log_locator.svg'], style='mpl20', remove_text=False)
+def test_log_locator_levels(split_collections):
 
     fig, ax = plt.subplots()
 
@@ -150,9 +179,12 @@ def test_log_locator_levels():
     cb = fig.colorbar(c, ax=ax)
     assert_array_almost_equal(cb.ax.get_yticks(), c.levels)
 
+    _maybe_split_collections(split_collections)
 
+
+@pytest.mark.parametrize("split_collections", [False, True])
 @image_comparison(['contour_datetime_axis.png'], style='mpl20')
-def test_contour_datetime_axis():
+def test_contour_datetime_axis(split_collections):
     fig = plt.figure()
     fig.subplots_adjust(hspace=0.4, top=0.98, bottom=.15)
     base = datetime.datetime(2013, 1, 1)
@@ -175,11 +207,13 @@ def test_contour_datetime_axis():
             label.set_ha('right')
             label.set_rotation(30)
 
+    _maybe_split_collections(split_collections)
 
+
+@pytest.mark.parametrize("split_collections", [False, True])
 @image_comparison(['contour_test_label_transforms.png'],
-                  remove_text=True, style='mpl20',
-                  tol=0 if platform.machine() == 'x86_64' else 0.08)
-def test_labels():
+                  remove_text=True, style='mpl20', tol=1.1)
+def test_labels(split_collections):
     # Adapted from pylab_examples example code: contour_demo.py
     # see issues #2475, #2843, and #2818 for explanation
     delta = 0.025
@@ -206,11 +240,38 @@ def test_labels():
     for x, y in disp_units:
         CS.add_label_near(x, y, inline=True, transform=False)
 
+    _maybe_split_collections(split_collections)
 
-@image_comparison(['contour_corner_mask_False.png',
-                   'contour_corner_mask_True.png'],
-                  remove_text=True)
-def test_corner_mask():
+
+def test_label_contour_start():
+    # Set up data and figure/axes that result in automatic labelling adding the
+    # label to the start of a contour
+
+    _, ax = plt.subplots(dpi=100)
+    lats = lons = np.linspace(-np.pi / 2, np.pi / 2, 50)
+    lons, lats = np.meshgrid(lons, lats)
+    wave = 0.75 * (np.sin(2 * lats) ** 8) * np.cos(4 * lons)
+    mean = 0.5 * np.cos(2 * lats) * ((np.sin(2 * lats)) ** 2 + 2)
+    data = wave + mean
+
+    cs = ax.contour(lons, lats, data)
+
+    with mock.patch.object(
+            cs, '_split_path_and_get_label_rotation',
+            wraps=cs._split_path_and_get_label_rotation) as mocked_splitter:
+        # Smoke test that we can add the labels
+        cs.clabel(fontsize=9)
+
+    # Verify at least one label was added to the start of a contour.  I.e. the
+    # splitting method was called with idx=0 at least once.
+    idxs = [cargs[0][1] for cargs in mocked_splitter.call_args_list]
+    assert 0 in idxs
+
+
+@pytest.mark.parametrize("split_collections", [False, True])
+@image_comparison(['contour_corner_mask_False.png', 'contour_corner_mask_True.png'],
+                  remove_text=True, tol=1.88)
+def test_corner_mask(split_collections):
     n = 60
     mask_level = 0.95
     noise_amp = 1.0
@@ -224,6 +285,8 @@ def test_corner_mask():
         plt.figure()
         plt.contourf(z, corner_mask=corner_mask)
 
+    _maybe_split_collections(split_collections)
+
 
 def test_contourf_decreasing_levels():
     # github issue 5477.
@@ -278,10 +341,11 @@ def test_clabel_zorder(use_clabeltext, contour_zorder, clabel_zorder):
 # tol because ticks happen to fall on pixel boundaries so small
 # floating point changes in tick location flip which pixel gets
 # the tick.
+@pytest.mark.parametrize("split_collections", [False, True])
 @image_comparison(['contour_log_extension.png'],
                   remove_text=True, style='mpl20',
                   tol=1.444)
-def test_contourf_log_extension():
+def test_contourf_log_extension(split_collections):
     # Remove this line when this test image is regenerated.
     plt.rcParams['pcolormesh.snap'] = False
 
@@ -313,14 +377,17 @@ def test_contourf_log_extension():
     assert_array_almost_equal_nulp(cb.ax.get_ylim(), np.array((1e-4, 1e6)))
     cb = plt.colorbar(c3, ax=ax3)
 
+    _maybe_split_collections(split_collections)
 
+
+@pytest.mark.parametrize("split_collections", [False, True])
 @image_comparison(
     ['contour_addlines.png'], remove_text=True, style='mpl20',
     tol=0.15 if platform.machine() in ('aarch64', 'ppc64le', 's390x')
         else 0.03)
 # tolerance is because image changed minutely when tick finding on
 # colorbars was cleaned up...
-def test_contour_addlines():
+def test_contour_addlines(split_collections):
     # Remove this line when this test image is regenerated.
     plt.rcParams['pcolormesh.snap'] = False
 
@@ -334,10 +401,13 @@ def test_contour_addlines():
     cb.add_lines(cont)
     assert_array_almost_equal(cb.ax.get_ylim(), [114.3091, 9972.30735], 3)
 
+    _maybe_split_collections(split_collections)
 
+
+@pytest.mark.parametrize("split_collections", [False, True])
 @image_comparison(baseline_images=['contour_uneven'],
                   extensions=['png'], remove_text=True, style='mpl20')
-def test_contour_uneven():
+def test_contour_uneven(split_collections):
     # Remove this line when this test image is regenerated.
     plt.rcParams['pcolormesh.snap'] = False
 
@@ -350,6 +420,8 @@ def test_contour_uneven():
     cs = ax.contourf(z, levels=[2, 4, 6, 10, 20])
     fig.colorbar(cs, ax=ax, spacing='uniform')
 
+    _maybe_split_collections(split_collections)
+
 
 @pytest.mark.parametrize(
     "rc_lines_linewidth, rc_contour_linewidth, call_linewidths, expected", [
@@ -365,7 +437,9 @@ def test_contour_linewidth(
         fig, ax = plt.subplots()
         X = np.arange(4*3).reshape(4, 3)
         cs = ax.contour(X, linewidths=call_linewidths)
-        assert cs.tlinewidths[0][0] == expected
+        assert cs.get_linewidths()[0] == expected
+        with pytest.warns(mpl.MatplotlibDeprecationWarning, match="tlinewidths"):
+            assert cs.tlinewidths[0][0] == expected
 
 
 @pytest.mark.backend("pdf")
@@ -374,9 +448,10 @@ def test_label_nonagg():
     plt.clabel(plt.contour([[1, 2], [3, 4]]))
 
 
+@pytest.mark.parametrize("split_collections", [False, True])
 @image_comparison(baseline_images=['contour_closed_line_loop'],
                   extensions=['png'], remove_text=True)
-def test_contour_closed_line_loop():
+def test_contour_closed_line_loop(split_collections):
     # github issue 19568.
     z = [[0, 0, 0], [0, 2, 0], [0, 0, 0], [2, 1, 2]]
 
@@ -385,6 +460,8 @@ def test_contour_closed_line_loop():
     ax.set_xlim(-0.1, 2.1)
     ax.set_ylim(-0.1, 3.1)
 
+    _maybe_split_collections(split_collections)
+
 
 def test_quadcontourset_reuse():
     # If QuadContourSet returned from one contour(f) call is passed as first
@@ -399,9 +476,10 @@ def test_quadcontourset_reuse():
     assert qcs3._contour_generator == qcs1._contour_generator
 
 
+@pytest.mark.parametrize("split_collections", [False, True])
 @image_comparison(baseline_images=['contour_manual'],
-                  extensions=['png'], remove_text=True)
-def test_contour_manual():
+                  extensions=['png'], remove_text=True, tol=0.89)
+def test_contour_manual(split_collections):
     # Manually specifying contour lines/polygons to plot.
     from matplotlib.contour import ContourSet
 
@@ -424,10 +502,13 @@ def test_contour_manual():
     ContourSet(ax, [2, 3], [segs], [kinds], filled=True, cmap=cmap)
     ContourSet(ax, [2], [segs], [kinds], colors='k', linewidths=3)
 
+    _maybe_split_collections(split_collections)
 
+
+@pytest.mark.parametrize("split_collections", [False, True])
 @image_comparison(baseline_images=['contour_line_start_on_corner_edge'],
                   extensions=['png'], remove_text=True)
-def test_contour_line_start_on_corner_edge():
+def test_contour_line_start_on_corner_edge(split_collections):
     fig, ax = plt.subplots(figsize=(6, 5))
 
     x, y = np.meshgrid([0, 1, 2, 3, 4], [0, 1, 2])
@@ -441,27 +522,31 @@ def test_contour_line_start_on_corner_edge():
     lines = ax.contour(x, y, z, corner_mask=True, colors='k')
     cbar.add_lines(lines)
 
+    _maybe_split_collections(split_collections)
+
 
 def test_find_nearest_contour():
     xy = np.indices((15, 15))
     img = np.exp(-np.pi * (np.sum((xy - 5)**2, 0)/5.**2))
     cs = plt.contour(img, 10)
 
-    nearest_contour = cs.find_nearest_contour(1, 1, pixel=False)
+    with pytest.warns(mpl._api.MatplotlibDeprecationWarning):
+        nearest_contour = cs.find_nearest_contour(1, 1, pixel=False)
     expected_nearest = (1, 0, 33, 1.965966, 1.965966, 1.866183)
     assert_array_almost_equal(nearest_contour, expected_nearest)
 
-    nearest_contour = cs.find_nearest_contour(8, 1, pixel=False)
+    with pytest.warns(mpl._api.MatplotlibDeprecationWarning):
+        nearest_contour = cs.find_nearest_contour(8, 1, pixel=False)
     expected_nearest = (1, 0, 5, 7.550173, 1.587542, 0.547550)
     assert_array_almost_equal(nearest_contour, expected_nearest)
 
-    nearest_contour = cs.find_nearest_contour(2, 5, pixel=False)
+    with pytest.warns(mpl._api.MatplotlibDeprecationWarning):
+        nearest_contour = cs.find_nearest_contour(2, 5, pixel=False)
     expected_nearest = (3, 0, 21, 1.884384, 5.023335, 0.013911)
     assert_array_almost_equal(nearest_contour, expected_nearest)
 
-    nearest_contour = cs.find_nearest_contour(2, 5,
-                                              indices=(5, 7),
-                                              pixel=False)
+    with pytest.warns(mpl._api.MatplotlibDeprecationWarning):
+        nearest_contour = cs.find_nearest_contour(2, 5, indices=(5, 7), pixel=False)
     expected_nearest = (5, 0, 16, 2.628202, 5.0, 0.394638)
     assert_array_almost_equal(nearest_contour, expected_nearest)
 
@@ -471,16 +556,16 @@ def test_find_nearest_contour_no_filled():
     img = np.exp(-np.pi * (np.sum((xy - 5)**2, 0)/5.**2))
     cs = plt.contourf(img, 10)
 
-    with pytest.raises(ValueError,
-                       match="Method does not support filled contours."):
+    with pytest.warns(mpl._api.MatplotlibDeprecationWarning), \
+         pytest.raises(ValueError, match="Method does not support filled contours."):
         cs.find_nearest_contour(1, 1, pixel=False)
 
-    with pytest.raises(ValueError,
-                       match="Method does not support filled contours."):
+    with pytest.warns(mpl._api.MatplotlibDeprecationWarning), \
+         pytest.raises(ValueError, match="Method does not support filled contours."):
         cs.find_nearest_contour(1, 10, indices=(5, 7), pixel=False)
 
-    with pytest.raises(ValueError,
-                       match="Method does not support filled contours."):
+    with pytest.warns(mpl._api.MatplotlibDeprecationWarning), \
+         pytest.raises(ValueError, match="Method does not support filled contours."):
         cs.find_nearest_contour(2, 5, indices=(2, 7), pixel=True)
 
 
@@ -518,7 +603,6 @@ def test_contourf_legend_elements():
 
 
 def test_contour_legend_elements():
-    from matplotlib.collections import LineCollection
     x = np.arange(1, 10)
     y = x.reshape(-1, 1)
     h = x * y
@@ -529,7 +613,7 @@ def test_contour_legend_elements():
                      extend='both')
     artists, labels = cs.legend_elements()
     assert labels == ['$x = 10.0$', '$x = 30.0$', '$x = 50.0$']
-    assert all(isinstance(a, LineCollection) for a in artists)
+    assert all(isinstance(a, mpl.lines.Line2D) for a in artists)
     assert all(same_color(a.get_color(), c)
                for a, c in zip(artists, colors))
 
@@ -567,9 +651,10 @@ def test_algorithm_supports_corner_mask(algorithm):
             plt.contourf(z, algorithm=algorithm, corner_mask=True)
 
 
+@pytest.mark.parametrize("split_collections", [False, True])
 @image_comparison(baseline_images=['contour_all_algorithms'],
-                  extensions=['png'], remove_text=True)
-def test_all_algorithms():
+                  extensions=['png'], remove_text=True, tol=0.06)
+def test_all_algorithms(split_collections):
     algorithms = ['mpl2005', 'mpl2014', 'serial', 'threaded']
 
     rng = np.random.default_rng(2981)
@@ -585,6 +670,8 @@ def test_all_algorithms():
         ax.contour(x, y, z, algorithm=algorithm, colors='k')
         ax.set_title(algorithm)
 
+    _maybe_split_collections(split_collections)
+
 
 def test_subfigure_clabel():
     # Smoke test for gh#23173
@@ -693,6 +780,21 @@ def test_contour_remove():
     assert ax.get_children() == orig_children
 
 
+def test_contour_no_args():
+    fig, ax = plt.subplots()
+    data = [[0, 1], [1, 0]]
+    with pytest.raises(TypeError, match=r"contour\(\) takes from 1 to 4"):
+        ax.contour(Z=data)
+
+
+def test_contour_clip_path():
+    fig, ax = plt.subplots()
+    data = [[0, 1], [1, 0]]
+    circle = mpatches.Circle([0.5, 0.5], 0.5, transform=ax.transAxes)
+    cs = ax.contour(data, clip_path=circle)
+    assert cs.get_clip_path() is not None
+
+
 def test_bool_autolevel():
     x, y = np.random.rand(2, 9)
     z = (np.arange(9) % 2).reshape((3, 3)).astype(bool)
@@ -708,3 +810,30 @@ def test_bool_autolevel():
     assert plt.tricontour(x, y, z).levels.tolist() == [.5]
     assert plt.tricontourf(x, y, z.tolist()).levels.tolist() == [0, .5, 1]
     assert plt.tricontourf(x, y, z).levels.tolist() == [0, .5, 1]
+
+
+def test_all_nan():
+    x = np.array([[np.nan, np.nan], [np.nan, np.nan]])
+    assert_array_almost_equal(plt.contour(x).levels,
+                              [-1e-13, -7.5e-14, -5e-14, -2.4e-14, 0.0,
+                                2.4e-14, 5e-14, 7.5e-14, 1e-13])
+
+
+def test_deprecated_apis():
+    cs = plt.contour(np.arange(16).reshape((4, 4)))
+    with pytest.warns(mpl.MatplotlibDeprecationWarning, match="collections"):
+        colls = cs.collections
+    with pytest.warns(PendingDeprecationWarning, match="allsegs"):
+        assert cs.allsegs == [p.vertices for c in colls for p in c.get_paths()]
+    with pytest.warns(PendingDeprecationWarning, match="allkinds"):
+        assert cs.allkinds == [p.codes for c in colls for p in c.get_paths()]
+    with pytest.warns(mpl.MatplotlibDeprecationWarning, match="tcolors"):
+        assert_array_equal(cs.tcolors, [c.get_edgecolor() for c in colls])
+    with pytest.warns(mpl.MatplotlibDeprecationWarning, match="tlinewidths"):
+        assert cs.tlinewidths == [c.get_linewidth() for c in colls]
+    with pytest.warns(mpl.MatplotlibDeprecationWarning, match="antialiased"):
+        assert cs.antialiased
+    with pytest.warns(mpl.MatplotlibDeprecationWarning, match="antialiased"):
+        cs.antialiased = False
+    with pytest.warns(mpl.MatplotlibDeprecationWarning, match="antialiased"):
+        assert not cs.antialiased
