@@ -25,7 +25,7 @@ def download_or_cache(url, version):
     if cache_dir is not None:  # Try to read from cache.
         try:
             data = (cache_dir / version).read_bytes()
-        except IOError:
+        except OSError:
             pass
         else:
             return BytesIO(data)
@@ -40,7 +40,7 @@ def download_or_cache(url, version):
             cache_dir.mkdir(parents=True, exist_ok=True)
             with open(cache_dir / version, "xb") as fout:
                 fout.write(data)
-        except IOError:
+        except OSError:
             pass
 
     return BytesIO(data)
@@ -63,7 +63,6 @@ def _get_xdg_cache_dir():
 
 if __name__ == "__main__":
     data = {
-        "v3.7.4": "10152802",
         "v3.7.3": "8336761",
         "v3.7.2": "8118151",
         "v3.7.1": "7697899",
@@ -123,7 +122,7 @@ if __name__ == "__main__":
     target_dir.mkdir(exist_ok=True, parents=True)
     header = []
     footer = []
-    with open(citing, "r") as fin:
+    with open(citing) as fin:
         target = header
         for ln in fin:
             if target is not None:
@@ -133,7 +132,7 @@ if __name__ == "__main__":
                 target = None
             if ln.strip() == ".. END OF AUTOGENERATED":
                 target = footer
-                target.append(ln)
+                target.append(ln.rstrip())
 
     with open(citing, "w") as fout:
         fout.write("\n".join(header))
@@ -152,4 +151,4 @@ if __name__ == "__main__":
             )
         fout.write("\n\n")
         fout.write("\n".join(footer))
-        fout.write('\n')
+        fout.write("\n")
